/*!
 * @file INSARIANNE.h
 *
 * This is a library for the BMP180 Barometric Pressure + Temp
 * sensor and the MUP6050 accelerometer, gyroscopique and temperature sensor
 *
 * These displays use I2C to communicate, 2 pins are required to
 * interface
 * 
 */

#ifndef INSARIANNE_H
#define INSARIANNE_H

#include <Arduino.h>
#include <Wire.h>


#define VERSION_LIB "1.0.1"


class I2C {
  protected:
    I2C(TwoWire *theWire = &Wire, HardwareSerial *theSer = &Serial);
    virtual void begin(uint8_t address);

    bool write(uint8_t data, uint8_t reg_data);
    uint8_t read8(uint8_t reg);
    uint16_t read16(uint8_t reg);
    bool read_n(uint8_t reg, uint8_t data[], int n);
    bool write_bits(uint8_t data, uint8_t reg, uint8_t bits, uint8_t shift);
    uint8_t read_bits(uint8_t reg, uint8_t bits, uint8_t shift);

    uint8_t _addr;
    TwoWire *_wire;
    HardwareSerial *_ser;
};


class BMP180 : private I2C
{
  public:
    BMP180();

    bool begin(void);

    void read_sensor(void);
    void Set_SLP(float altitude_base);

    float pressure, sealevelpressure, temperature, altitude;

  private:
    void Temperature(void);
    void Pressure(void);
    void Altitude(void);

    uint8_t oversampling;
    int16_t ac1, ac2, ac3, b1, b2, mb, mc, md;
    uint16_t ac4, ac5, ac6;
    int32_t UP, UT;
    int32_t c3, c5, c6, x1, x2, x3;
    uint32_t c4, c7, c1;
};


class MPU6050 : private I2C
{
  public:
    MPU6050();

    bool begin();
    bool begin(uint8_t para_gyr, uint8_t para_acc);
    bool read_sensor(void);

    bool read_acce(void);
    bool read_gyro(void);
    bool read_temp(void);

    void Set_gyro_scale(float new_scale);
    void Set_accel_scale(float new_scale);
    bool Set_param_register(uint8_t val, uint8_t reg);
    bool Set_param_register_bits(uint8_t val, uint8_t reg, uint8_t bits, uint8_t shift);
    float Get_gyro_scale(void);
    float Get_accel_scale(void);
    uint8_t Get_param_register(uint8_t reg);
    uint8_t Get_param_register(uint8_t reg, uint8_t bits, uint8_t shift);

    void SetOffset_zero(void);
    void CalcOffset(void);

    uint8_t test(void);

    float temperature, gyroX, gyroY, gyroZ, accX, accY, accZ;
    float accXoffset, accYoffset, accZoffset;
    float gyroXoffset, gyroYoffset, gyroZoffset;

  private:
    int _sensorID_tem = 0x650, _sensorID_acc = 0x651, _sensorID_gyr = 0x652;
    int16_t rawAccX, rawAccY, rawAccZ, rawTemp, rawGyroX, rawGyroY, rawGyroZ;
    float accel_scale, gyro_scale;
};

#endif 
